@@ -4,7 +4,7 @@
 def test_function():
     pass
 
-class TestClass:
+class TestClass(object):
     def test_method(self):
         pass
     def test_anothermethod(self):
