@@ -1,3 +1,4 @@
+# -*- coding: utf-8 -*-
 from __future__ import absolute_import, division, print_function
 import pytest
 import os
@@ -120,6 +121,16 @@ def test_makepyfile_unicode(testdir):
     testdir.makepyfile(unichr(0xfffd))
 
 
+def test_makepyfile_utf8(testdir):
+    """Ensure makepyfile accepts utf-8 bytes as input (#2738)"""
+    utf8_contents = u"""
+        def setup_function(function):
+            mixed_encoding = u'São Paulo'
+    """.encode('utf-8')
+    p = testdir.makepyfile(utf8_contents)
+    assert u"mixed_encoding = u'São Paulo'".encode('utf-8') in p.read('rb')
+
+
 def test_inline_run_clean_modules(testdir):
     test_mod = testdir.makepyfile("def test_foo(): assert True")
     result = testdir.inline_run(str(test_mod))
@@ -130,7 +141,7 @@ def test_inline_run_clean_modules(testdir):
     assert result2.ret == EXIT_TESTSFAILED
 
 
-def test_assert_outcomes_after_pytest_erro(testdir):
+def test_assert_outcomes_after_pytest_error(testdir):
     testdir.makepyfile("def test_foo(): assert True")
 
     result = testdir.runpytest('--unexpected-argument')
