@@ -589,7 +589,7 @@ class TestSkipif(object):
             @pytest.mark.skipif("hasattr(os, 'sep')")
             def test_func():
                 pass
-        """)  # noqa
+        """)
         x = pytest.raises(pytest.skip.Exception, lambda:
                           pytest_runtest_setup(item))
         assert x.value.msg == "condition: hasattr(os, 'sep')"
@@ -663,7 +663,7 @@ def test_skipif_class(testdir):
 
 
 def test_skip_reasons_folding():
-    path = 'xyz'
+    path = "xyz"
     lineno = 3
     message = "justso"
     longrepr = (path, lineno, message)
@@ -676,13 +676,19 @@ def test_skip_reasons_folding():
     ev1.longrepr = longrepr
 
     ev2 = X()
+    ev2.when = "execute"
     ev2.longrepr = longrepr
     ev2.skipped = True
 
-    l = folded_skips([ev1, ev2])
-    assert len(l) == 1
-    num, fspath, lineno, reason = l[0]
-    assert num == 2
+    # ev3 might be a collection report
+    ev3 = X()
+    ev3.longrepr = longrepr
+    ev3.skipped = True
+
+    values = folded_skips([ev1, ev2, ev3])
+    assert len(values) == 1
+    num, fspath, lineno, reason = values[0]
+    assert num == 3
     assert fspath == path
     assert lineno == lineno
     assert reason == message
@@ -713,6 +719,27 @@ def test_skipped_reasons_functional(testdir):
     assert result.ret == 0
 
 
+def test_skipped_folding(testdir):
+    testdir.makepyfile(
+        test_one="""
+            import pytest
+            pytestmark = pytest.mark.skip("Folding")
+            def setup_function(func):
+                pass
+            def test_func():
+                pass
+            class TestClass(object):
+                def test_method(self):
+                    pass
+       """,
+    )
+    result = testdir.runpytest('-rs')
+    result.stdout.fnmatch_lines([
+        "*SKIP*2*test_one.py: Folding"
+    ])
+    assert result.ret == 0
+
+
 def test_reportchars(testdir):
     testdir.makepyfile("""
         import pytest
@@ -983,6 +1010,40 @@ def test_module_level_skip_error(testdir):
     )
 
 
+def test_module_level_skip_with_allow_module_level(testdir):
+    """
+    Verify that using pytest.skip(allow_module_level=True) is allowed
+    """
+    testdir.makepyfile("""
+        import pytest
+        pytest.skip("skip_module_level", allow_module_level=True)
+
+        def test_func():
+            assert 0
+    """)
+    result = testdir.runpytest("-rxs")
+    result.stdout.fnmatch_lines(
+        "*SKIP*skip_module_level"
+    )
+
+
+def test_invalid_skip_keyword_parameter(testdir):
+    """
+    Verify that using pytest.skip() with unknown parameter raises an error
+    """
+    testdir.makepyfile("""
+        import pytest
+        pytest.skip("skip_module_level", unknown=1)
+
+        def test_func():
+            assert 0
+    """)
+    result = testdir.runpytest()
+    result.stdout.fnmatch_lines(
+        "*TypeError:*['unknown']*"
+    )
+
+
 def test_mark_xfail_item(testdir):
     # Ensure pytest.mark.xfail works with non-Python Item
     testdir.makeconftest("""
